@@ -22,8 +22,10 @@
 #include <system.h>
 
 #include <fnmatch.h>
+#include <getline.h>
 #include <argp.h>
 #include <argp-namefrob.h>
+#include <argp-fmtstream.h>
 
 #include <signal.h>
 #if ! defined SIGCHLD && defined SIGCLD
@@ -216,12 +218,12 @@ subcommand_string (enum subcommand c)
 }
 
 void
-tar_list_quoting_styles (FILE *fp, char *prefix)
+tar_list_quoting_styles (argp_fmtstream_t fs, char *prefix)
 {
   int i;
 
   for (i = 0; quoting_style_args[i]; i++)
-    fprintf (fp, "%s%s\n", prefix, quoting_style_args[i]);
+    argp_fmtstream_printf (fs, "%s%s\n", prefix, quoting_style_args[i]);
 }
 
 void
@@ -248,7 +250,6 @@ enum
   ATIME_PRESERVE_OPTION,
   BACKUP_OPTION,
   CHECKPOINT_OPTION,
-  CHECK_LINKS_OPTION,
   DELAY_DIRECTORY_RESTORE_OPTION,
   DELETE_OPTION,
   EXCLUDE_CACHES_OPTION,
@@ -262,6 +263,7 @@ enum
   INDEX_FILE_OPTION,
   KEEP_NEWER_FILES_OPTION,
   MODE_OPTION,
+  MTIME_OPTION,
   NEWER_MTIME_OPTION,
   NO_ANCHORED_OPTION,
   NO_DELAY_DIRECTORY_RESTORE_OPTION,
@@ -280,6 +282,7 @@ enum
   OCCURRENCE_OPTION,
   OLD_ARCHIVE_OPTION,
   ONE_FILE_SYSTEM_OPTION,
+  OVERWRITE_DIR_OPTION,
   OVERWRITE_OPTION,
   OWNER_OPTION,
   PAX_OPTION,
@@ -297,12 +300,14 @@ enum
   SAME_OWNER_OPTION,
   SHOW_DEFAULTS_OPTION,
   SHOW_OMITTED_DIRS_OPTION,
-  SHOW_STORED_NAMES_OPTION,
+  SHOW_TRANSFORMED_NAMES_OPTION,
+  SPARSE_VERSION_OPTION,
   STRIP_COMPONENTS_OPTION,
   SUFFIX_OPTION,
   TEST_LABEL_OPTION,
   TOTALS_OPTION,
   TO_COMMAND_OPTION,
+  TRANSFORM_OPTION,
   UNQUOTE_OPTION,
   USAGE_OPTION,
   USE_COMPRESS_PROGRAM_OPTION,
@@ -315,13 +320,16 @@ enum
 
 const char *argp_program_version = "tar (" PACKAGE_NAME ") " VERSION;
 const char *argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";
-static char doc[] = N_("GNU `tar' saves many files together into a single tape or disk archive, and can restore individual files from the archive.\n\
+static char const doc[] = N_("\
+GNU `tar' saves many files together into a single tape or disk archive, \
+and can restore individual files from the archive.\n\
 \n\
 Examples:\n\
   tar -cf archive.tar foo bar  # Create archive.tar from files foo and bar.\n\
   tar -tvf archive.tar         # List all files in archive.tar verbosely.\n\
-  tar -xf archive.tar          # Extract all files from archive.tar.\n\
-\vThe backup suffix is `~', unless set with --suffix or SIMPLE_BACKUP_SUFFIX.\n\
+  tar -xf archive.tar          # Extract all files from archive.tar.\n")
+"\v"
+N_("The backup suffix is `~', unless set with --suffix or SIMPLE_BACKUP_SUFFIX.\n\
 The version control may be set with --backup or VERSION_CONTROL, values are:\n\n\
   none, off       never make backups\n\
   t, numbered     make numbered backups\n\
@@ -369,7 +377,7 @@ static struct argp_option options[] = {
   {"delete", DELETE_OPTION, 0, 0,
    N_("delete from the archive (not on mag tapes!)"), GRID+1 },
   {"test-label", TEST_LABEL_OPTION, NULL, 0,
-   N_("Test archive volume label and exit"), GRID+1 },
+   N_("test the archive volume label and exit"), GRID+1 },
 #undef GRID
 
 #define GRID 20
@@ -378,6 +386,8 @@ static struct argp_option options[] = {
 
   {"sparse", 'S', 0, 0,
    N_("handle sparse files efficiently"), GRID+1 },
+  {"sparse-version", SPARSE_VERSION_OPTION, N_("MAJOR[.MINOR]"), 0,
+   N_("set version of the sparse format to use (implies --sparse)"), GRID+1},
   {"incremental", 'G', 0, 0,
    N_("handle old GNU-format incremental backup"), GRID+1 },
   {"listed-incremental", 'g', N_("FILE"), 0,
@@ -385,14 +395,18 @@ static struct argp_option options[] = {
   {"ignore-failed-read", IGNORE_FAILED_READ_OPTION, 0, 0,
    N_("do not exit with nonzero on unreadable files"), GRID+1 },
   {"occurrence", OCCURRENCE_OPTION, N_("NUMBER"), OPTION_ARG_OPTIONAL,
-   N_("process only the NUMBERth occurrence of each file in the archive. This option is valid only in conjunction with one of the subcommands --delete, --diff, --extract or --list and when a list of files is given either on the command line or via -T option. NUMBER defaults to 1."), GRID+1 },
+   N_("process only the NUMBERth occurrence of each file in the archive;"
+      " this option is valid only in conjunction with one of the subcommands"
+      " --delete, --diff, --extract or --list and when a list of files"
+      " is given either on the command line or via the -T option;"
+      " NUMBER defaults to 1"), GRID+1 },
   {"seek", 'n', NULL, 0,
    N_("archive is seekable"), GRID+1 },
 #undef GRID
 
 #define GRID 30
   {NULL, 0, NULL, 0,
-   N_("Overwrite control:\n"), GRID+1 },
+   N_("Overwrite control:"), GRID+1 },
 
   {"verify", 'W', 0, 0,
    N_("attempt to verify the archive after writing it"), GRID+1 },
@@ -410,6 +424,9 @@ static struct argp_option options[] = {
    N_("empty hierarchies prior to extracting directory"), GRID+1 },
   {"no-overwrite-dir", NO_OVERWRITE_DIR_OPTION, 0, 0,
    N_("preserve metadata of existing directories"), GRID+1 },
+  {"overwrite-dir", OVERWRITE_DIR_OPTION, 0, 0,
+   N_("overwrite metadata of existing directories when extracting (default)"),
+   GRID+1 },
 #undef GRID
 
 #define GRID 40
@@ -434,6 +451,8 @@ static struct argp_option options[] = {
    N_("force NAME as owner for added files"), GRID+1 },
   {"group", GROUP_OPTION, N_("NAME"), 0,
    N_("force NAME as group for added files"), GRID+1 },
+  {"mtime", MTIME_OPTION, N_("DATE-OR-FILE"), 0,
+   N_("set mtime for added files from DATE-OR-FILE"), GRID+1 },
   {"mode", MODE_OPTION, N_("CHANGES"), 0,
    N_("force (symbolic) mode CHANGES for added files"), GRID+1 },
   {"atime-preserve", ATIME_PRESERVE_OPTION,
@@ -461,9 +480,10 @@ static struct argp_option options[] = {
   {"preserve", PRESERVE_OPTION, 0, 0,
    N_("same as both -p and -s"), GRID+1 },
   {"delay-directory-restore", DELAY_DIRECTORY_RESTORE_OPTION, 0, 0,
-   N_("Delay setting modification times and permissions of extracted directories until the end of extraction."), GRID+1 },
+   N_("delay setting modification times and permissions of extracted"
+      " directories until the end of extraction"), GRID+1 },
   {"no-delay-directory-restore", NO_DELAY_DIRECTORY_RESTORE_OPTION, 0, 0,
-   N_("Cancel the effect of --delay-directory-restore option."), GRID+1 },
+   N_("cancel the effect of --delay-directory-restore option"), GRID+1 },
 #undef GRID
 
 #define GRID 60
@@ -524,7 +544,7 @@ static struct argp_option options[] = {
    N_("Archive format selection:"), GRID },
 
   {"format", 'H', N_("FORMAT"), 0,
-   N_("create archive of the given format."), GRID+1 },
+   N_("create archive of the given format"), GRID+1 },
 
   {NULL, 0, NULL, 0, N_("FORMAT is one of the following:"), GRID+2 },
   {"  v7", 0, NULL, OPTION_DOC|OPTION_NO_TRANS, N_("old V7 tar format"),
@@ -544,10 +564,10 @@ static struct argp_option options[] = {
   {"portability", 0, 0, OPTION_ALIAS, NULL, GRID+8 },
   {"posix", POSIX_OPTION, 0, 0,
    N_("same as --format=posix"), GRID+8 },
-  {"pax-option", PAX_OPTION, N_("keyword[[:]=value][,keyword[[:]=value], ...]"), 0,
+  {"pax-option", PAX_OPTION, N_("keyword[[:]=value][,keyword[[:]=value]]..."), 0,
    N_("control pax keywords"), GRID+8 },
   {"label", 'V', N_("TEXT"), 0,
-   N_("create archive with volume name TEXT. At list/extract time, use TEXT as a globbing pattern for volume name"), GRID+8 },
+   N_("create archive with volume name TEXT; at list/extract time, use TEXT as a globbing pattern for volume name"), GRID+8 },
   {"bzip2", 'j', 0, 0,
    N_("filter the archive through bzip2"), GRID+8 },
   {"gzip", 'z', 0, 0,
@@ -583,23 +603,10 @@ static struct argp_option options[] = {
    N_("exclude patterns listed in FILE"), GRID+1 },
   {"exclude-caches", EXCLUDE_CACHES_OPTION, 0, 0,
    N_("exclude directories containing a cache tag"), GRID+1 },
-  {"ignore-case", IGNORE_CASE_OPTION, 0, 0,
-   N_("exclusion ignores case"), GRID+1 },
-  {"anchored", ANCHORED_OPTION, 0, 0,
-   N_("exclude patterns match file name start"), GRID+1 },
-  {"no-anchored", NO_ANCHORED_OPTION, 0, 0,
-   N_("exclude patterns match after any `/' (default)"), GRID+1 },
-  {"no-ignore-case", NO_IGNORE_CASE_OPTION, 0, 0,
-   N_("exclusion is case sensitive (default)"), GRID+1 },
-  {"no-wildcards", NO_WILDCARDS_OPTION, 0, 0,
-   N_("exclude patterns are plain strings"), GRID+1 },
-  {"no-wildcards-match-slash", NO_WILDCARDS_MATCH_SLASH_OPTION, 0, 0,
-   N_("exclude pattern wildcards do not match `/'"), GRID+1 },
   {"no-recursion", NO_RECURSION_OPTION, 0, 0,
    N_("avoid descending automatically in directories"), GRID+1 },
   {"one-file-system", ONE_FILE_SYSTEM_OPTION, 0, 0,
    N_("stay in local file system when creating archive"), GRID+1 },
-  {NULL, 'l', 0, OPTION_HIDDEN, "", GRID+1 },
   {"recursion", RECURSION_OPTION, 0, 0,
    N_("recurse into directories (default)"), GRID+1 },
   {"absolute-names", 'P', 0, 0,
@@ -608,22 +615,47 @@ static struct argp_option options[] = {
    N_("follow symlinks; archive and dump the files they point to"), GRID+1 },
   {"starting-file", 'K', N_("MEMBER-NAME"), 0,
    N_("begin at member MEMBER-NAME in the archive"), GRID+1 },
-  {"strip-components", STRIP_COMPONENTS_OPTION, N_("NUMBER"), 0,
-   N_("strip NUMBER leading components from file names"), GRID+1 },
   {"newer", 'N', N_("DATE-OR-FILE"), 0,
    N_("only store files newer than DATE-OR-FILE"), GRID+1 },
+  {"after-date", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
   {"newer-mtime", NEWER_MTIME_OPTION, N_("DATE"), 0,
    N_("compare date and time when data changed only"), GRID+1 },
-  {"after-date", 'N', N_("DATE"), 0,
-   N_("same as -N"), GRID+1 },
   {"backup", BACKUP_OPTION, N_("CONTROL"), OPTION_ARG_OPTIONAL,
    N_("backup before removal, choose version CONTROL"), GRID+1 },
   {"suffix", SUFFIX_OPTION, N_("STRING"), 0,
    N_("backup before removal, override usual suffix ('~' unless overridden by environment variable SIMPLE_BACKUP_SUFFIX)"), GRID+1 },
+#undef GRID
+
+#define GRID 92
+  {NULL, 0, NULL, 0,
+   N_("File name transformations:"), GRID },
+  {"strip-components", STRIP_COMPONENTS_OPTION, N_("NUMBER"), 0,
+   N_("strip NUMBER leading components from file names on extraction"),
+   GRID+1 },
+  {"transform", TRANSFORM_OPTION, N_("EXPRESSION"), 0,
+   N_("use sed replace EXPRESSION to transform file names"), GRID+1 },
+#undef GRID
+
+#define GRID 95
+  {NULL, 0, NULL, 0,
+   N_("File name matching options (affect both exclude and include patterns):"),
+   GRID },
+  {"ignore-case", IGNORE_CASE_OPTION, 0, 0,
+   N_("ignore case"), GRID+1 },
+  {"anchored", ANCHORED_OPTION, 0, 0,
+   N_("patterns match file name start"), GRID+1 },
+  {"no-anchored", NO_ANCHORED_OPTION, 0, 0,
+   N_("patterns match after any `/' (default for exclusion)"), GRID+1 },
+  {"no-ignore-case", NO_IGNORE_CASE_OPTION, 0, 0,
+   N_("case sensitive matching (default)"), GRID+1 },
   {"wildcards", WILDCARDS_OPTION, 0, 0,
-   N_("exclude patterns use wildcards (default)"), GRID+1 },
+   N_("use wildcards (default for exclusion)"), GRID+1 },
+  {"no-wildcards", NO_WILDCARDS_OPTION, 0, 0,
+   N_("verbatim string matching"), GRID+1 },
+  {"no-wildcards-match-slash", NO_WILDCARDS_MATCH_SLASH_OPTION, 0, 0,
+   N_("wildcards do not match `/'"), GRID+1 },
   {"wildcards-match-slash", WILDCARDS_MATCH_SLASH_OPTION, 0, 0,
-   N_("exclude pattern wildcards match `/' (default)"), GRID+1 },
+   N_("wildcards match `/' (default for exclusion)"), GRID+1 },
 #undef GRID
 
 #define GRID 100
@@ -632,12 +664,16 @@ static struct argp_option options[] = {
 
   {"verbose", 'v', 0, 0,
    N_("verbosely list files processed"), GRID+1 },
-  {"checkpoint", CHECKPOINT_OPTION, 0, 0,
-   N_("display progress messages every 10th record"), GRID+1 },
-  {"check-links", CHECK_LINKS_OPTION, 0, 0,
+  {"checkpoint", CHECKPOINT_OPTION, N_("[.]NUMBER"), OPTION_ARG_OPTIONAL,
+   N_("display progress messages every NUMBERth record (default 10)"),
+   GRID+1 },
+  {"check-links", 'l', 0, 0,
    N_("print a message if not all links are dumped"), GRID+1 },
-  {"totals", TOTALS_OPTION, 0, 0,
-   N_("print total bytes written while creating archive"), GRID+1 },
+  {"totals", TOTALS_OPTION, N_("SIGNAL"), OPTION_ARG_OPTIONAL,
+   N_("print total bytes after processing the archive; "
+      "with an argument - print total bytes when this SIGNAL is delivered; "
+      "Allowed signals are: SIGHUP, SIGQUIT, SIGINT, SIGUSR1 and SIGUSR2; "
+      "the names without SIG prefix are also accepted"), GRID+1 },
   {"utc", UTC_OPTION, 0, 0,
    N_("print file modification dates in UTC"), GRID+1 },
   {"index-file", INDEX_FILE_OPTION, N_("FILE"), 0,
@@ -648,18 +684,19 @@ static struct argp_option options[] = {
    N_("ask for confirmation for every action"), GRID+1 },
   {"confirmation", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
   {"show-defaults", SHOW_DEFAULTS_OPTION, 0, 0,
-   N_("Show tar defaults"), GRID+1 },
+   N_("show tar defaults"), GRID+1 },
   {"show-omitted-dirs", SHOW_OMITTED_DIRS_OPTION, 0, 0,
-   N_("When listing or extracting, list each directory that does not match search criteria"), GRID+1 },
-  {"show-stored-names", SHOW_STORED_NAMES_OPTION, 0, 0,
-   N_("When creating archive in verbose mode, list member names as stored in the archive"),
+   N_("when listing or extracting, list each directory that does not match search criteria"), GRID+1 },
+  {"show-transformed-names", SHOW_TRANSFORMED_NAMES_OPTION, 0, 0,
+   N_("show file or archive names after transformation"),
    GRID+1 },
+  {"show-stored-names", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
   {"quoting-style", QUOTING_STYLE_OPTION, N_("STYLE"), 0,
-   N_("Set name quoting style. See below for valid STYLE values."), GRID+1 },
+   N_("set name quoting style; see below for valid STYLE values"), GRID+1 },
   {"quote-chars", QUOTE_CHARS_OPTION, N_("STRING"), 0,
-   N_("Additionally quote characters from STRING"), GRID+1 },
+   N_("additionally quote characters from STRING"), GRID+1 },
   {"no-quote-chars", NO_QUOTE_CHARS_OPTION, N_("STRING"), 0,
-   N_("Disable quoting for characters from STRING"), GRID+1 },
+   N_("disable quoting for characters from STRING"), GRID+1 },
 #undef GRID
 
 #define GRID 110
@@ -667,7 +704,7 @@ static struct argp_option options[] = {
    N_("Compatibility options:"), GRID },
 
   {NULL, 'o', 0, 0,
-   N_("when creating, same as --old-archive. When extracting, same as --no-same-owner"), GRID+1 },
+   N_("when creating, same as --old-archive; when extracting, same as --no-same-owner"), GRID+1 },
 #undef GRID
 
 #define GRID 120
@@ -675,15 +712,15 @@ static struct argp_option options[] = {
    N_("Other options:"), GRID },
 
   {"restrict", RESTRICT_OPTION, 0, 0,
-   N_("Restrict use of some potentially harmful options"), -1 },
+   N_("disable use of some potentially harmful options"), -1 },
 
-  {"help",  '?', 0, 0,  N_("Give this help list"), -1},
-  {"usage", USAGE_OPTION, 0, 0,  N_("Give a short usage message"), -1},
-  {"version", VERSION_OPTION, 0, 0,  N_("Print program version"), -1},
+  {"help",  '?', 0, 0,  N_("give this help list"), -1},
+  {"usage", USAGE_OPTION, 0, 0,  N_("give a short usage message"), -1},
+  {"version", VERSION_OPTION, 0, 0,  N_("print program version"), -1},
   /* FIXME -V (--label) conflicts with the default short option for
      --version */
   {"HANG",	  HANG_OPTION,    "SECS", OPTION_ARG_OPTIONAL | OPTION_HIDDEN,
-   N_("Hang for SECS seconds (default 3600)"), 0},
+   N_("hang for SECS seconds (default 3600)"), 0},
 #undef GRID
 
   {0, 0, 0, 0, 0, 0}
@@ -693,37 +730,85 @@ static char const *const atime_preserve_args[] =
 {
   "replace", "system", NULL
 };
+
 static enum atime_preserve const atime_preserve_types[] =
 {
   replace_atime_preserve, system_atime_preserve
 };
+
+/* Make sure atime_preserve_types has as much entries as atime_preserve_args
+   (minus 1 for NULL guard) */
 ARGMATCH_VERIFY (atime_preserve_args, atime_preserve_types);
 
+/* Wildcard matching settings */
+enum wildcards
+  {
+    default_wildcards, /* For exclusion == enable_wildcards,
+			  for inclusion == disable_wildcards */
+    disable_wildcards,
+    enable_wildcards
+  };
 
-struct tar_args {
-  char const *textual_date_option;
-  int exclude_options;
-  bool o_option;
-  int pax_option;
-  char const *backup_suffix_string;
-  char const *version_control_string;
-  int input_files;
+struct tar_args        /* Variables used during option parsing */
+{
+  struct textual_date *textual_date; /* Keeps the arguments to --newer-mtime
+					and/or --date option if they are
+					textual dates */
+  enum wildcards wildcards;        /* Wildcard settings (--wildcards/
+				      --no-wildcards) */
+  int matching_flags;              /* exclude_fnmatch options */
+  int include_anchored;            /* Pattern anchoring options used for
+				      file inclusion */
+  bool o_option;                   /* True if -o option was given */
+  bool pax_option;                 /* True if --pax-option was given */
+  char const *backup_suffix_string;   /* --suffix option argument */
+  char const *version_control_string; /* --backup option argument */
+  bool input_files;                /* True if some input files where given */
 };
 
-static void
-show_default_settings (FILE *stream)
-{
-  fprintf (stream,
-	   "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s",
-	   archive_format_string (DEFAULT_ARCHIVE_FORMAT),
-	   DEFAULT_ARCHIVE, DEFAULT_BLOCKING,
-	   quoting_style_args[DEFAULT_QUOTING_STYLE],
-	   DEFAULT_RMT_COMMAND);
+#define MAKE_EXCL_OPTIONS(args) \
+ ((((args)->wildcards != disable_wildcards) ? EXCLUDE_WILDCARDS : 0) \
+  | (args)->matching_flags \
+  | recursion_option)
+
+#define MAKE_INCL_OPTIONS(args) \
+ ((((args)->wildcards == enable_wildcards) ? EXCLUDE_WILDCARDS : 0) \
+  | (args)->include_anchored \
+  | (args)->matching_flags \
+  | recursion_option)
+
 #ifdef REMOTE_SHELL
-  fprintf (stream, " --rsh-command=%s", REMOTE_SHELL);
-#endif
-  fprintf (stream, "\n");
+# define DECL_SHOW_DEFAULT_SETTINGS(stream, printer)                      \
+{                                                                         \
+  printer (stream,                                                        \
+	   "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s",   \
+	   archive_format_string (DEFAULT_ARCHIVE_FORMAT),                \
+	   DEFAULT_ARCHIVE, DEFAULT_BLOCKING,                             \
+	   quoting_style_args[DEFAULT_QUOTING_STYLE],                     \
+	   DEFAULT_RMT_COMMAND);                                          \
+  printer (stream, " --rsh-command=%s", REMOTE_SHELL);                    \
+  printer (stream, "\n");                                                 \
 }
+#else
+# define DECL_SHOW_DEFAULT_SETTINGS(stream, printer)                      \
+{                                                                         \
+  printer (stream,                                                        \
+	   "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s",   \
+	   archive_format_string (DEFAULT_ARCHIVE_FORMAT),                \
+	   DEFAULT_ARCHIVE, DEFAULT_BLOCKING,                             \
+	   quoting_style_args[DEFAULT_QUOTING_STYLE],                     \
+	   DEFAULT_RMT_COMMAND);                                          \
+  printer (stream, "\n");                                                 \
+}
+#endif
+
+static void
+show_default_settings (FILE *fp)
+     DECL_SHOW_DEFAULT_SETTINGS(fp, fprintf)
+
+static void
+show_default_settings_fs (argp_fmtstream_t fs)
+     DECL_SHOW_DEFAULT_SETTINGS(fs, argp_fmtstream_printf)
 
 static void
 set_subcommand_option (enum subcommand subcommand)
@@ -745,7 +830,123 @@ set_use_compress_program_option (const char *string)
 
   use_compress_program_option = string;
 }
+
+static RETSIGTYPE
+sigstat (int signo)
+{
+  compute_duration ();
+  print_total_stats ();
+#ifndef HAVE_SIGACTION
+  signal (signo, sigstat);
+#endif
+}
 
+static void
+stat_on_signal (int signo)
+{
+#ifdef HAVE_SIGACTION
+  struct sigaction act;
+  act.sa_handler = sigstat;
+  sigemptyset (&act.sa_mask);
+  act.sa_flags = 0;
+  sigaction (signo, &act, NULL);
+#else
+  signal (signo, sigstat);
+#endif
+}
+
+void
+set_stat_signal (const char *name)
+{
+  static struct sigtab
+  {
+    char *name;
+    int signo;
+  } sigtab[] = {
+    { "SIGUSR1", SIGUSR1 },
+    { "USR1", SIGUSR1 },
+    { "SIGUSR2", SIGUSR2 },
+    { "USR2", SIGUSR2 },
+    { "SIGHUP", SIGHUP },
+    { "HUP", SIGHUP },
+    { "SIGINT", SIGINT },
+    { "INT", SIGINT },
+    { "SIGQUIT", SIGQUIT },
+    { "QUIT", SIGQUIT }
+  };
+  struct sigtab *p;
+
+  for (p = sigtab; p < sigtab + sizeof (sigtab) / sizeof (sigtab[0]); p++)
+    if (strcmp (p->name, name) == 0)
+      {
+	stat_on_signal (p->signo);
+	return;
+      }
+  FATAL_ERROR ((0, 0, _("Unknown signal name: %s"), name));
+}
+
+
+struct textual_date
+{
+  struct textual_date *next;
+  struct timespec *ts;
+  const char *option;
+  const char *date;
+};
+
+static void
+get_date_or_file (struct tar_args *args, const char *option,
+		  const char *str, struct timespec *ts)
+{
+  if (FILE_SYSTEM_PREFIX_LEN (str) != 0
+      || ISSLASH (*str)
+      || *str == '.')
+    {
+      struct stat st;
+      if (deref_stat (dereference_option, str, &st) != 0)
+	{
+	  stat_error (str);
+	  USAGE_ERROR ((0, 0, _("Date sample file not found")));
+	}
+      *ts = get_stat_mtime (&st);
+    }
+  else
+    {
+      if (! get_date (ts, str, NULL))
+	{
+	  WARN ((0, 0, _("Substituting %s for unknown date format %s"),
+		 tartime (*ts, false), quote (str)));
+	  ts->tv_nsec = 0;
+	}
+      else
+	{
+	  struct textual_date *p = xmalloc (sizeof (*p));
+	  p->ts = ts;
+	  p->option = option;
+	  p->date = str;
+	  p->next = args->textual_date;
+	  args->textual_date = p;
+	}
+    }
+}
+
+static void
+report_textual_dates (struct tar_args *args)
+{
+  struct textual_date *p;
+  for (p = args->textual_date; p; )
+    {
+      struct textual_date *next = p->next;
+      char const *treated_as = tartime (*p->ts, true);
+      if (strcmp (p->date, treated_as) != 0)
+	WARN ((0, 0, _("Option %s: Treating date `%s' as %s"),
+	       p->option, p->date, treated_as));
+      free (p);
+      p = next;
+    }
+}
+
+
 static volatile int _argp_hang;
 
 enum read_file_list_state  /* Result of reading file name from the list file */
@@ -914,6 +1115,28 @@ update_argv (const char *filename, struct argp_state *state)
     }
 }
 
+
+static void
+tar_help (struct argp_state *state)
+{
+  argp_fmtstream_t fs;
+  state->flags |= ARGP_NO_EXIT;
+  argp_state_help (state, state->out_stream,
+		   ARGP_HELP_STD_HELP & ~ARGP_HELP_BUG_ADDR);
+  /* FIXME: use struct uparams.rmargin (from argp-help.c) instead of 79 */
+  fs = argp_make_fmtstream (state->out_stream, 0, 79, 0);
+
+  argp_fmtstream_printf (fs, "\n%s\n\n",
+		       _("Valid arguments for --quoting-style options are:"));
+  tar_list_quoting_styles (fs, "  ");
+
+  argp_fmtstream_puts (fs, _("\n*This* tar defaults to:\n"));
+  show_default_settings_fs (fs);
+  argp_fmtstream_putc (fs, '\n');
+  argp_fmtstream_printf (fs, _("Report bugs to %s.\n"),
+			 argp_program_bug_address);
+  argp_fmtstream_free (fs);
+}
 
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
@@ -924,8 +1147,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
     {
       case ARGP_KEY_ARG:
 	/* File name or non-parsed option, because of ARGP_IN_ORDER */
-	name_add (arg);
-	args->input_files++;
+	name_add_name (arg, MAKE_INCL_OPTIONS (args));
+	args->input_files = true;
 	break;
 
     case 'A':
@@ -960,8 +1183,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'C':
-      name_add ("-C");
-      name_add (arg);
+      name_add_dir (arg);
       break;
 
     case 'd':
@@ -970,12 +1192,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'f':
       if (archive_names == allocated_archive_names)
-	{
-	  allocated_archive_names *= 2;
-	  archive_name_array =
-	    xrealloc (archive_name_array,
-		      sizeof (const char *) * allocated_archive_names);
-	}
+	archive_name_array = x2nrealloc (archive_name_array,
+					 &allocated_archive_names,
+					 sizeof (archive_name_array[0]));
+
       archive_name_array[archive_names++] = arg;
       break;
 
@@ -1033,23 +1253,16 @@ parse_opt (int key, char *arg, struct argp_state *state)
       addname (arg, 0);
       break;
 
-    case 'l':
-      /* Historically equivalent to --one-file-system. This usage is
-	 incompatible with UNIX98 and POSIX specs and therefore is
-	 deprecated. The semantics of -l option will be changed in
-	 future versions. See TODO.
-      */
-      WARN ((0, 0,
-	     _("Semantics of -l option will change in the future releases.")));
-      WARN ((0, 0,
-	     _("Please use --one-file-system option instead.")));
-      /* FALL THROUGH */
     case ONE_FILE_SYSTEM_OPTION:
       /* When dumping directories, don't dump files/subdirectories
 	 that are on other filesystems. */
       one_file_system_option = true;
       break;
 
+    case 'l':
+      check_links_option = 1;
+      break;
+
     case 'L':
       {
 	uintmax_t u;
@@ -1072,6 +1285,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       multi_volume_option = true;
       break;
 
+    case MTIME_OPTION:
+      get_date_or_file (args, "--mtime", arg, &mtime_option);
+      set_mtime_option = true;
+      break;
+
     case 'n':
       seekable_archive = true;
       break;
@@ -1083,31 +1301,9 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case NEWER_MTIME_OPTION:
       if (NEWER_OPTION_INITIALIZED (newer_mtime_option))
 	USAGE_ERROR ((0, 0, _("More than one threshold date")));
-
-      if (FILE_SYSTEM_PREFIX_LEN (arg) != 0
-	  || ISSLASH (*arg)
-	  || *arg == '.')
-	{
-	  struct stat st;
-	  if (deref_stat (dereference_option, arg, &st) != 0)
-	    {
-	      stat_error (arg);
-	      USAGE_ERROR ((0, 0, _("Date sample file not found")));
-	    }
-	  newer_mtime_option = get_stat_mtime (&st);
-	}
-      else
-	{
-	  if (! get_date (&newer_mtime_option, arg, NULL))
-	    {
-	      WARN ((0, 0, _("Substituting %s for unknown date format %s"),
-		     tartime (newer_mtime_option, false), quote (arg)));
-	      newer_mtime_option.tv_nsec = 0;
-	    }
-	  else
-	    args->textual_date_option = arg;
-	}
-
+      get_date_or_file (args,
+			key == NEWER_MTIME_OPTION ? "--newer-mtime"
+			  : "--after-date", arg, &newer_mtime_option);
       break;
 
     case 'o':
@@ -1134,7 +1330,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       /* Print block numbers for debugging bad tar archives.  */
 
       /* It would surely make sense to exchange -B and -R, but it seems
-	 that -B has been used for a long while in Sun tar ans most
+	 that -B has been used for a long while in Sun tar and most
 	 BSD-derived systems.  This is a consequence of the block/record
 	 terminology confusion.  */
 
@@ -1142,7 +1338,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 's':
-      /* Names to extr are sorted.  */
+      /* Names to extract are sorted.  */
 
       same_order_option = true;
       break;
@@ -1151,6 +1347,22 @@ parse_opt (int key, char *arg, struct argp_state *state)
       sparse_option = true;
       break;
 
+    case SPARSE_VERSION_OPTION:
+      sparse_option = true;
+      {
+	char *p;
+	tar_sparse_major = strtoul (arg, &p, 10);
+	if (*p)
+	  {
+	    if (*p != '.')
+	      USAGE_ERROR ((0, 0, _("Invalid sparse version value")));
+	    tar_sparse_minor = strtoul (p + 1, &p, 10);
+	    if (*p)
+	      USAGE_ERROR ((0, 0, _("Invalid sparse version value")));
+	  }
+      }
+      break;
+
     case 't':
       set_subcommand_option (LIST_SUBCOMMAND);
       verbose_option++;
@@ -1203,7 +1415,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'X':
       if (add_exclude_file (add_exclude, excluded, arg,
-			    args->exclude_options | recursion_option, '\n')
+			    MAKE_EXCL_OPTIONS (args), '\n')
 	  != 0)
 	{
 	  int e = errno;
@@ -1220,7 +1432,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case ANCHORED_OPTION:
-      args->exclude_options |= EXCLUDE_ANCHORED;
+      args->matching_flags |= EXCLUDE_ANCHORED;
       break;
 
     case ATIME_PRESERVE_OPTION:
@@ -1236,7 +1448,22 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case CHECKPOINT_OPTION:
-      checkpoint_option = true;
+      if (arg)
+	{
+	  char *p;
+
+	  if (*arg == '.')
+	    {
+	      checkpoint_style = checkpoint_dot;
+	      arg++;
+	    }
+	  checkpoint_option = strtoul (arg, &p, 0);
+	  if (*p)
+	    FATAL_ERROR ((0, 0,
+			  _("--checkpoint value is not an integer")));
+	}
+      else
+	checkpoint_option = 10;
       break;
 
     case BACKUP_OPTION:
@@ -1258,7 +1485,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case EXCLUDE_OPTION:
-      add_exclude (excluded, arg, args->exclude_options | recursion_option);
+      add_exclude (excluded, arg, MAKE_EXCL_OPTIONS (args));
       break;
 
     case EXCLUDE_CACHES_OPTION:
@@ -1278,7 +1505,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case IGNORE_CASE_OPTION:
-      args->exclude_options |= FNM_CASEFOLD;
+      args->matching_flags |= FNM_CASEFOLD;
       break;
 
     case IGNORE_COMMAND_ERROR_OPTION:
@@ -1316,11 +1543,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case NO_ANCHORED_OPTION:
-      args->exclude_options &= ~ EXCLUDE_ANCHORED;
+      args->include_anchored = 0; /* Clear the default for comman line args */
+      args->matching_flags &= ~ EXCLUDE_ANCHORED;
       break;
 
     case NO_IGNORE_CASE_OPTION:
-      args->exclude_options &= ~ FNM_CASEFOLD;
+      args->matching_flags &= ~ FNM_CASEFOLD;
       break;
 
     case NO_IGNORE_COMMAND_ERROR_OPTION:
@@ -1337,11 +1565,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case NO_WILDCARDS_OPTION:
-      args->exclude_options &= ~ EXCLUDE_WILDCARDS;
+      args->wildcards = disable_wildcards;
       break;
 
     case NO_WILDCARDS_MATCH_SLASH_OPTION:
-      args->exclude_options |= FNM_FILE_NAME;
+      args->matching_flags |= FNM_FILE_NAME;
       break;
 
     case NULL_OPTION:
@@ -1366,6 +1594,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
 
+    case OVERWRITE_DIR_OPTION:
+      old_files_option = DEFAULT_OLD_FILES;
+      break;
+
     case OVERWRITE_OPTION:
       old_files_option = OVERWRITE_OLD_FILES;
       break;
@@ -1394,7 +1626,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case PAX_OPTION:
-      args->pax_option++;
+      args->pax_option = true;
       xheader_set_option (arg);
       break;
 
@@ -1403,6 +1635,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case PRESERVE_OPTION:
+      /* FIXME: What it is good for? */
       same_permissions_option = true;
       same_order_option = true;
       break;
@@ -1462,8 +1695,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
       show_omitted_dirs_option = true;
       break;
 
-    case SHOW_STORED_NAMES_OPTION:
-      show_stored_names_option = true;
+    case SHOW_TRANSFORMED_NAMES_OPTION:
+      show_transformed_names_option = true;
       break;
 
     case SUFFIX_OPTION:
@@ -1478,7 +1711,14 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case TOTALS_OPTION:
-      totals_option = true;
+      if (arg)
+	set_stat_signal (arg);
+      else
+	totals_option = true;
+      break;
+
+    case TRANSFORM_OPTION:
+      set_transform_expr (arg);
       break;
 
     case USE_COMPRESS_PROGRAM_OPTION:
@@ -1490,15 +1730,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case WILDCARDS_OPTION:
-      args->exclude_options |= EXCLUDE_WILDCARDS;
+      args->wildcards = enable_wildcards;
       break;
 
     case WILDCARDS_MATCH_SLASH_OPTION:
-      args->exclude_options &= ~ FNM_FILE_NAME;
-      break;
-
-    case CHECK_LINKS_OPTION:
-      check_links_option = 1;
+      args->matching_flags &= ~ FNM_FILE_NAME;
       break;
 
     case NO_RECURSION_OPTION:
@@ -1579,12 +1815,9 @@ parse_opt (int key, char *arg, struct argp_state *state)
 #endif /* not DENSITY_LETTER */
 
 	if (archive_names == allocated_archive_names)
-	  {
-	    allocated_archive_names *= 2;
-	    archive_name_array =
-	      xrealloc (archive_name_array,
-			sizeof (const char *) * allocated_archive_names);
-	  }
+	  archive_name_array = x2nrealloc (archive_name_array,
+					   &allocated_archive_names,
+					   sizeof (archive_name_array[0]));
 	archive_name_array[archive_names++] = xstrdup (buf);
       }
       break;
@@ -1597,18 +1830,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 #endif /* not DEVICE_PREFIX */
 
     case '?':
-      state->flags |= ARGP_NO_EXIT;
-      argp_state_help (state, state->out_stream,
-		       ARGP_HELP_STD_HELP & ~ARGP_HELP_BUG_ADDR);
-      fprintf (state->out_stream, "\n%s\n\n",
-	       _("Valid arguments for --quoting-style options are:"));
-      tar_list_quoting_styles (state->out_stream, "  ");
-
-      fprintf (state->out_stream, _("\n*This* tar defaults to:\n"));
-      show_default_settings (state->out_stream);
-      fprintf (state->out_stream, "\n");
-      fprintf (state->out_stream, _("Report bugs to %s.\n"),
-	       argp_program_bug_address);
+      tar_help (state);
       close_stdout ();
       exit (0);
 
@@ -1676,13 +1898,15 @@ decode_options (int argc, char **argv)
   struct tar_args args;
 
   /* Set some default option values.  */
-  args.textual_date_option = NULL;
-  args.exclude_options = EXCLUDE_WILDCARDS;
-  args.o_option = 0;
-  args.pax_option = 0;
+  args.textual_date = NULL;
+  args.wildcards = default_wildcards;
+  args.matching_flags = 0;
+  args.include_anchored = EXCLUDE_ANCHORED;
+  args.o_option = false;
+  args.pax_option = false;
   args.backup_suffix_string = getenv ("SIMPLE_BACKUP_SUFFIX");
   args.version_control_string = 0;
-  args.input_files = 0;
+  args.input_files = false;
 
   subcommand_option = UNKNOWN_SUBCOMMAND;
   archive_format = DEFAULT_FORMAT;
@@ -1693,6 +1917,8 @@ decode_options (int argc, char **argv)
   newer_mtime_option.tv_nsec = -1;
   recursion_option = FNM_LEADING_DIR;
   unquote_option = true;
+  tar_sparse_major = 1;
+  tar_sparse_minor = 0;
 
   owner_option = -1;
   group_option = -1;
@@ -1789,10 +2015,14 @@ decode_options (int argc, char **argv)
   /* Handle operands after any "--" argument.  */
   for (; index < argc; index++)
     {
-      name_add (argv[index]);
-      args.input_files++;
+      name_add_name (argv[index], MAKE_INCL_OPTIONS (&args));
+      args.input_files = true;
     }
 
+  /* Warn about implicit use of the wildcards in command line arguments.
+     See TODO */
+  warn_regex_usage = args.wildcards == default_wildcards;
+
   /* Derive option values and check option consistency.  */
 
   if (archive_format == DEFAULT_FORMAT)
@@ -1811,13 +2041,6 @@ decode_options (int argc, char **argv)
 		   | FORMAT_MASK (GNU_FORMAT)
 		   | FORMAT_MASK (POSIX_FORMAT));
 
-  if (multi_volume_option
-      && archive_format == POSIX_FORMAT
-      && subcommand_option == CREATE_SUBCOMMAND
-      && !tape_length_option)
-    USAGE_ERROR ((0, 0,
-		  _("creating multi-volume archives in posix format requires using --tape-length (-L) option")));
-
   if (occurrence_option)
     {
       if (!args.input_files)
@@ -1930,7 +2153,7 @@ decode_options (int argc, char **argv)
     {
       /* --test-label is silent if the user has specified the label name to
 	 compare against. */
-      if (args.input_files == 0)
+      if (!args.input_files)
 	verbose_option++;
     }
   else if (utc_option)
@@ -1942,7 +2165,7 @@ decode_options (int argc, char **argv)
   switch (subcommand_option)
     {
     case CREATE_SUBCOMMAND:
-      if (args.input_files == 0 && !files_from_option)
+      if (!args.input_files && !files_from_option)
 	USAGE_ERROR ((0, 0,
 		      _("Cowardly refusing to create an empty archive")));
       break;
@@ -1971,6 +2194,16 @@ decode_options (int argc, char **argv)
       break;
     }
 
+  /* Initialize stdlis */
+  if (index_file_name)
+    {
+      stdlis = fopen (index_file_name, "w");
+      if (! stdlis)
+	open_error (index_file_name);
+    }
+  else
+    stdlis = to_stdout_option ? stderr : stdout;
+
   archive_name_cursor = archive_name_array;
 
   /* Prepare for generating backup names.  */
@@ -1987,13 +2220,8 @@ decode_options (int argc, char **argv)
 	backup_option = false;
     }
 
-  if (verbose_option && args.textual_date_option)
-    {
-      char const *treated_as = tartime (newer_mtime_option, true);
-      if (strcmp (args.textual_date_option, treated_as) != 0)
-	WARN ((0, 0, _("Treating date `%s' as %s"),
-	       args.textual_date_option, treated_as));
-    }
+  if (verbose_option)
+    report_textual_dates (&args);
 }
 
 
@@ -2018,6 +2246,9 @@ main (int argc, char **argv)
   /* Make sure we have first three descriptors available */
   stdopen ();
 
+  /* Close all inherited open descriptors, except for the first three */
+  closeopen ();
+
   /* Pre-allocate a few structures.  */
 
   allocated_archive_names = 10;
@@ -2032,8 +2263,6 @@ main (int argc, char **argv)
   signal (SIGCHLD, SIG_DFL);
 #endif
 
-  init_names ();
-
   /* Decode options.  */
 
   decode_options (argc, argv);
@@ -2062,8 +2291,6 @@ main (int argc, char **argv)
 
     case CREATE_SUBCOMMAND:
       create_archive ();
-      if (totals_option)
-	print_total_written ();
       break;
 
     case EXTRACT_SUBCOMMAND:
@@ -2086,6 +2313,9 @@ main (int argc, char **argv)
       break;
     }
 
+  if (totals_option)
+    print_total_stats ();
+
   if (check_links_option)
     check_links ();
 
